﻿using Milimoe.FunGame.Core.Api.Utility;
using Milimoe.FunGame.Core.Entity;
using Milimoe.FunGame.Core.Library.Constant;

namespace Milimoe.FunGame.Testing.Skills
{
    public class 变幻之心 : Skill
    {
        public override long Id => 3005;
        public override string Name => "变幻之心";
        public override string Description => Effects.Count > 0 ? Effects.First().Description : "";
        public override double EPCost => 100;
        public override double CD => 30;
        public override double HardnessTime => 10;

        public 变幻之心(Character character) : base(SkillType.SuperSkill, character)
        {
            Effects.Add(new 变幻之心特效(this));
        }
    }

    public class 变幻之心特效(Skill skill) : Effect(skill)
    {
        public override long Id => Skill.Id;
        public override string Name => "变幻之心";
        public override string Description => $"检查 [ 智慧与力量 ] 的模式。在力量模式下，立即回复 {生命值回复 * 100}% 生命值；智力模式下，下一次魔法伤害提升 {伤害提升 * 100}%。";
        public override bool TargetSelf => true;

        private double 生命值回复 => Calculation.Round4Digits(0.25 + 0.03 * (Level - 1));
        private double 伤害提升 => Calculation.Round4Digits(0.55 + 0.25 * (Level - 1));

        public override void OnEffectGained(Character character)
        {
            Skill.IsInEffect = true;
        }

        public override void OnEffectLost(Character character)
        {
            Skill.IsInEffect = false;
        }

        public override void AlterExpectedDamageBeforeCalculation(Character character, Character enemy, ref double damage, bool isNormalAttack, bool isMagicDamage, MagicType magicType)
        {
            if (character == Skill.Character && isMagicDamage)
            {
                double 实际伤害提升百分比 = 伤害提升;
                double 实际伤害提升 = Calculation.Round2Digits(damage * 实际伤害提升百分比);
                damage = Calculation.Round2Digits(damage + 实际伤害提升);
                WriteLine("[ " + character + " ] 发动了变幻之心！伤害提升了 " + 实际伤害提升 + " 点！");
                character.Effects.Remove(this);
                OnEffectLost(character);
            }
        }

        public override void OnSkillCasted(Character caster, List<Character> enemys, List<Character> teammates, Dictionary<string, object> others)
        {
            IEnumerable<Effect> effects = caster.Effects.Where(e => e is 智慧与力量特效);
            if (effects.Any())
            {
                if (caster.PrimaryAttribute == PrimaryAttribute.STR)
                {
                    double 回复的生命 = Calculation.Round2Digits(生命值回复 * caster.MaxHP);
                    caster.HP += 回复的生命;
                    WriteLine("[ " + caster + " ] 回复了 " + 回复的生命 + " 点生命值！");
                }
                else if (caster.PrimaryAttribute == PrimaryAttribute.INT)
                {
                    if (!caster.Effects.Contains(this))
                    {
                        caster.Effects.Add(this);
                        OnEffectGained(caster);
                    }
                }
            }
        }
    }
}
